from typing import Optional

from pydantic import BaseModel, Field

from util.wapi_util import execute_wapi_function


class Bldg(BaseModel):
    gpk: str = Field()
    name: Optional[str] = Field(default=None)
    number: Optional[str] = Field(default=None)
    postal_code: Optional[int] = Field(default=None)
    site_fq_name: Optional[str] = Field(default=None)
    street: Optional[str] = Field(default=None)


class Room(BaseModel):
    bldg: Optional[str] = Field(default=None)
    description: Optional[str] = Field(default=None)
    floor: Optional[str] = Field(default=None)
    gpk: str = Field()
    name: Optional[str] = Field(default=None)
    number: Optional[str] = Field(default=None)


class Module(BaseModel):
    bldg: Optional[str] = Field(default=None)
    fq_name: Optional[str] = Field(default=None)
    gpk: str = Field()
    room: Optional[str] = Field(default=None)
    label: Optional[str] = Field(default=None)
    type: Optional[str] = Field(default=None)


class Port(BaseModel):
    gpk: str = Field()
    mdl_bldg: Optional[str] = Field(default=None)
    mdl_fq_name: Optional[str] = Field(default=None)
    mdl_room: Optional[str] = Field(default=None)
    name: Optional[str] = Field(default=None)


class Site(BaseModel):
    gpk: str = Field()
    description: Optional[str] = Field(default=None)
    fq_name: Optional[str] = Field(default=None)
    name: Optional[str] = Field(default=None)
    parent_fq_name: Optional[str] = Field(default=None)


class BCD(BaseModel):
    gpk: Optional[str] = Field(default=None)
    name: Optional[str] = Field(default=None)
    categ: Optional[str] = Field(default=None)
    is_own: Optional[bool] = Field(default=None)
    seclvl: Optional[int] = Field(default=None)
    description: Optional[str] = Field(default=None)
    dhcp_domain: Optional[str] = Field(default=None)  # TODO: check in db
    dhcp_enabled: Optional[bool] = Field(default=None)
    admin_decription: Optional[str] = Field(default=None)  # TODO: check in db
    dhcp_opt_t1_percent: Optional[int] = Field(default=None)
    dhcp_opt_t2_percent: Optional[int] = Field(default=None)
    dhcp_accept_new_leases: Optional[bool] = Field(default=None)
    dhcp_ttl_value_percent: Optional[int] = Field(default=None)
    dhcp_leasetime_dyn_days: Optional[int] = Field(default=None)
    dhcp_leasetime_dyn_hours: Optional[int] = Field(default=None)
    dhcp_leasetime_dyn_minutes: Optional[int] = Field(default=None)
    dhcp_leasetime_static_days: Optional[int] = Field(default=None)
    dhcp_leasetime_static_hours: Optional[int] = Field(default=None)
    dhcp_leasetime_static_minutes: Optional[int] = Field(default=None)
    dhcp_offer_rsv_time_minutes: Optional[int] = Field(default=None)
    dhcp_offer_rsv_time_seconds: Optional[int] = Field(default=None)
    dhcp_accept_requested_hostname: Optional[bool] = Field(default=None)
    dhcp_dynamic_lease_removal_time: Optional[str] = Field(default=None)
    dhcp_new_leases_get_static_addr: Optional[bool] = Field(default=None)
    dhcp_new_leases_get_static_fqdn: Optional[bool] = Field(default=None)

    @classmethod
    def get_by_name(cls, conn, user: str, name: str):
        result = execute_wapi_function(conn, [
            {'idx': 'bcd_list', 'name': 'nd.bcd.list', 'old': {'name': name}},
        ], user=user)
        bcd_list: list = result['bcd_list']
        if len(bcd_list) == 0:
            return None

        return cls(**bcd_list[0])


class VLAN(BaseModel):
    id: Optional[int] = Field()
    bcd: Optional[str] = Field()
    gpk: Optional[str] = Field()
    name: Optional[str] = Field()
    net_instnc: Optional[str] = Field()
    description: Optional[str] = Field()

    @classmethod
    def get_by_bcd(cls, conn, user: str, bcd: str) -> list:
        result = execute_wapi_function(conn, [
            {'name': 'nd.vlan.list', 'old': {'bcd': bcd}}
        ], user=user)
        ret = []
        for vlan in result['nd_vlan_list']:
            ret.append(cls(**vlan))
        return ret
